@@ -8,6 +8,7 @@ use MooseX::NonMoose;
 use namespace::autoclean;
 use JSON;
 use Data::Munge qw/elem/;
+use Scalar::Util qw/reftype/;
 extends 'DBIx::Class::Core';
 
 __PACKAGE__->load_components("InflateColumn::DateTime", "TimeStamp");
@@ -83,10 +84,7 @@ sub convert_to {
     my $self = shift;
     my ($type) = @_;
 
-    die "Cannot convert new row"
-        if ! $self->in_storage;
-
-    die "Cannot convert to $type"
+    die "Cannot convert " . $self->name . " to $type"
         unless elem $type, $self->viable_type_conversions;
 
     return $self->decoded_value
@@ -106,6 +104,35 @@ sub convert_to {
     $conv->{$key}->($self->decoded_value);
 }
 
+sub set_type_from_value {
+    my $self = shift;
+    my $value = shift // $self->decoded_value;
+
+    if (ref $value) {
+        if (ref $value =~ /Bool/) {
+            # JSON::Boolean, JSON::PP::Boolean, etc
+            $self->data_type('bool')
+        }
+        elsif (reftype $value eq 'HASH') {
+            $self->data_type('object');
+        }
+        elsif (reftype $value eq 'ARRAY') {
+            $self->data_type('array');
+        }
+        else {
+            warn "Cannot determine type for " . $self->name . " given " . reftype $value . ".";
+        }
+    }
+    else {
+        if ($value =~ /\n/) {
+            $self->data_type('textarea');
+        }
+        else {
+            $self->data_type('text');
+        }
+    }
+}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -122,7 +149,7 @@ OpusVL::SysParams::Schema::Result::SysInfo
 
 =head1 VERSION
 
-version 0.14
+version 0.15
 
 =head1 NAME
 
@@ -161,10 +188,30 @@ returns the current type.
 For a new row, this simply returns the whole set, because we haven't specified
 the type yet.
 
+=head2 METHODS
+
 =head2 convert_to
 
-Converts the value to the provided data type (see C<viable_type_conversions>).
-If it's already that type, returns the decoded value.
+=over
+
+=item $data_type
+
+=back
+
+Converts the value to the provided data type (see C<viable_type_conversions>),
+if necessary. Returns the decoded value, i.e. a Perl data structure.
+
+=head2 set_type_from_value
+
+=over
+
+=item $value
+
+=back
+
+Attempts to guess the data type of the provided value, which defaults to the
+row's value if not provided. Sets the C<data_type> property on the field, but
+doesn't save it.
 
 =head1 AUTHOR
 
