@@ -21,12 +21,22 @@ sub set
 	my $self  = shift;
 	my $name  = shift;
 	my $value = shift;
+    my $data_type = shift;
 
-	my $info = $self->update_or_create
-	({
-		name  => $name,
-		value => JSON->new->allow_nonref->encode($value)
-	});
+	my $info = $self->find_or_new({
+        name  => $name,
+    });
+
+    $info->set_column(value => JSON->new->allow_nonref->encode($value));
+    if ($data_type) {
+        $info->set_column(data_type => $data_type);
+    }
+
+    if (! $info->data_type) {
+        $info->set_type_from_value($value);
+    }
+
+    $info->update_or_insert;
 
 	return $value;
 }
@@ -78,7 +88,7 @@ OpusVL::SysParams::Schema::ResultSet::SysInfo
 
 =head1 VERSION
 
-version 0.14
+version 0.15
 
 =head1 SYNOPSIS
 
@@ -98,15 +108,29 @@ OpusVL::SysParams::Schema::ResultSet::SysInfo
 
 =head2 get
 
-Get a system parameter.  The key name is simply a string.  It's suggested you use some 
-kind of schema like 'system.key' to prevent name clashes with other unoriginal programmers.
+Get a system parameter. The key name will only be meaningful if the same string
+has already been provided to L</set> at some point in the past, or created via
+the L<OpusVL::AppKitX::SysParams> interface.
 
 =head2 set
 
+=over
+
+=item B<$name>
+
+=item B<$value>
+
+=item B<$data_type>
+
+=back
+
 Set a system parameter.  The key name is simply a string.  It's suggested you use some 
 kind of schema like 'system.key' to prevent name clashes with other unoriginal programmers.
 
-The value can be any data structure so long as it doesn't contain code.  
+The value and data type should correspond. A guess will be made, if you don't
+provide the data type. Any value that can be JSON-encoded should work (i.e. no
+CODE refs), but see L<OpusVL::SysParams::Schema::Result::SysInfo> for the list
+of options for data type, and hence value type.
 
 =head2 del
 
